/*

  Nullsoft WASABI Source File License

  Copyright 1999-2001 Nullsoft, Inc.

    This software is provided 'as-is', without any express or implied
    warranty.  In no event will the authors be held liable for any damages
    arising from the use of this software.

    Permission is granted to anyone to use this software for any purpose,
    including commercial applications, and to alter it and redistribute it
    freely, subject to the following restrictions:

    1. The origin of this software must not be misrepresented; you must not
       claim that you wrote the original software. If you use this software
       in a product, an acknowledgment in the product documentation would be
       appreciated but is not required.
    2. Altered source versions must be plainly marked as such, and must not be
       misrepresented as being the original software.
    3. This notice may not be removed or altered from any source distribution.


  Brennan Underwood
  brennan@nullsoft.com

*/

// ===========================================================================
//
//    NULLSOFT WASABI SDK EXAMPLE PROJECTS
//
//      File:     ExampleBFrameWnd.cpp
//
//!##   Purpose:  This source module contains the functions for the 
//!##             construction of the XML Document Display sheet of 
//!##             the ExampleB project.
//
//      Requires: Please read ExampleBWnd.cpp first.
//
//      Notes:    A note on the comments in this document:
//
//                Notes that begin with *** are important notes that everyone
//                needs to read.  The other comments assist readability or
//                explain the thinking behind sections of code which may not
//                be immediately obvious to the novice programmer.
//
//                Or I'm just typing to hear myself clickyclack.
//


//
//  BETA 1 RELEASE NOTE:
//
//    Yes, I'm aware of the fact that this tab doesn't work.  You think we'd hold up
//  the shipping of our entire Beta just for that?  Feh.  Just keep an eye out in the
//  next one and I'll try and document properly the changes I had to make to make
//  this tab work properly.
//                      
//          mig
//





// Always begin with including std.h
#include "../common/std.h"

#include "ExampleBWnd.h"

#include "../common/framewnd.h"
#include "../common/contwnd.h"
#include "../common/buttwnd.h"
#include "../common/textbar.h"
#include "../common/editwnd.h"
#include "../common/xlatstr.h"
#include "../common/treewnd.h"
#include "../common/urlgrab.h"

#include "../studio/xmlreader.h"
#include "../studio/xmlmanager.h"
#include "../studio/xmlmanagedobj.h"

//
// Constants and Globals.
const char * xmlTempFilename = "exb-temp.xml";

// ===========================================================================
//
//  EXAMPLEB: class ExampleBFrameXmlBase
//
// *** This object shows how to extend the TreeItem class to be fully and 
// automatically constructed to replicate the data hierarchy of an XML file.
//
// Neat, eh?
//
// Now do you understand why we used a template for XmlManager?
//
// Can you think of other GUI items that would be fun to extend to be created
// fully formed from an XML file?
class ExampleBFrameXmlBase : public TreeItem {
public:
  ExampleBFrameXmlBase(TreeWnd * theTree, ExampleBWnd * theDisplay) : 
      tree(theTree), display(theDisplay), TreeItem() {}

  //
  //  Here's a tree item method we override
  //
  virtual void onSelect() {
    display->displayText(data);
  }


  //
  //  And the rest of these are just overrides for the XmlManager
  //

  // It is important, however, that xmlTestPath create the proper new object.
  virtual ExampleBFrameXmlBase * xmlTestPath(const char * xmlpath, const char * xmltag) {
    // Here, we create a new version of this subclass and pass it back.
    ExampleBFrameXmlBase * newNode = new ExampleBFrameXmlBase(tree,display);
    // Set its label (this is bogus)
    newNode->setLabel(xmltag);
    // Attach our tree connections.
    tree->addTreeItem(newNode, this, FALSE, TRUE);
    // Make it the new active element.
    return newNode;
  }

  virtual void xmlSetTag(const char * xmlpath, const char * xmltag) {
  }

  virtual void xmlSetData(const char * xmlpath, const char * xmltag, const char * xmldata) {
    data = xmldata;
  }

  virtual void xmlSetParams(const char * xmlpath, const char * xmltag, XmlReaderParams * xmlparams) {
    params = *xmlparams; // note that we're assigning from one class type to another.  
  }

  virtual void xmlComplete(const char * xmlpath, const char * xmltag) {
    // We do nothing!  We don't care!  Hedonism, baby... pure hedonism.
  }
public:
  TreeWnd *     tree;
  ExampleBWnd * display;
  String        data;
  XmlStrParams  params;
};

// ===========================================================================
//
//  EXAMPLEB: "ExampleBWnd::createFramesSheet"
//
int ExampleBWnd::createFramesSheet() {  
  // Make some frames and process some XML.

  // Okay, so, frames are kewl, right?
  // Hellyar!  Frames are groovy!
  //      Let's start cooking on them!

  // Instantiate our top level window
  FrameWnd * horiz = new FrameWnd();
  horiz->setName(_("Frames"));  // Remember, the name of the object is the
                                // name on the tab in the stabsheet buttonbar.
  //
  //  *** Important Options for FrameWnd.
  //
  // Set some preinitialization parameters, like which way to divide,
  horiz->setDividerType(DIVIDER_HORIZONTAL);
  // Setting the object resizable makes a divider get created that's moveable
  horiz->setResizeable(TRUE);
  // And this simply allows us to set the divider position to whatever we'd like it to be.
  horiz->setDividerPos(SDP_FROMTOP, 50);

  //////////////////////////////////////////////
  // *** Setup the TOP frame to contain the GUI 
  //     to specify an url to an XML document.
  //////////////////////////////////////////////
  
    // First, we'll make a container window and stuff it on in.
    ContWnd *container = new ContWnd();
    // In it, we'll put:

    //
    // 1) A 'Go' button.
    //
    ButtonWnd *go = new ButtonWnd();
    go->setButtonText(_("Go"), 16);
    go->setButtonId(EXB_XML_GO);
    go->setNotifyWindow(this);
    container->addChild(go, EXB_LEFT_OFFSET, (EXB_TOP_OFFSET * 2),
      EXB_BUTTON_WIDTH, EXB_BUTTON_HEIGHT, 1);

    //
    // 2) A text entry field.
    //
    xmlurledit = new EditWnd();
    xmlurledit->setName(_("This is now a text edit field."));
    container->addChild(xmlurledit, 
      EXB_LEFT_OFFSET + EXB_BUTTON_WIDTH + EXB_BUTTON_SPACER,
      (EXB_TOP_OFFSET * 2), EXB_BUTTON_WIDTH * 6, EXB_BUTTON_HEIGHT, 1);

    //
    //  Now, after we add the text entry field, we fill it in with interesting data.
    STRNCPY(myXMLURLBuffer,"http://www.slashdot.org/slashdot.xml", EXB_XML_BUFFSIZE); // In this case, slashdot's XML headlines file.
    xmlurledit->setBuffer(myXMLURLBuffer, EXB_XML_BUFFSIZE);
    //
    // WIP: This is EXTRAORDINARILY clunky.  
    //
    // *** I'll be altering the EditWnd interface to better handle containing
    //  its own string data at some point, soon.  However, this "more raw"
    //  method of assigning the EditWnd to allow the client code to control
    //  the buffer being edited will remain (it is very useful for some
    //  complex forms of GUI design).

  //////////////////////////////////////////////
  // *** Setup the BOTTOM frame to have a 
  //     vertical frame with XML data display
  //////////////////////////////////////////////

    // Instantiate
    FrameWnd *vert = new FrameWnd();
    //
    //  Again, a framewind always has two children.  
    //  One on each side of the frame.          (duh!)
    //

    //
    // And set the frame options.
    vert->setDividerType(DIVIDER_VERTICAL);
    vert->setResizeable(TRUE);
    vert->setDividerPos(SDP_FROMLEFT, 50);

    //
    //  So now we make the child windows in prep for adding them to mr vert:

    // Make our tree.
    myFrameTreeWnd = new TreeWnd;

    ContWnd * cont = new ContWnd;

    myFrameTextBar = new TextBar;

    cont->addChild(myFrameTextBar, EXB_LEFT_OFFSET, 
      EXB_BUTTON_HEIGHT + EXB_BUTTON_SPACER,
      EXB_BUTTON_WIDTH * 6, EXB_BUTTON_HEIGHT, 0/*"Invalidate on Resize"*/);

    //
    // And we add them, or set them, or something like that.
    vert->setChildren(myFrameTreeWnd, cont);

  //////////////////////////////////////////////
  // *** Set the two children into the main obj
  //////////////////////////////////////////////

    horiz->setChildren(container, vert);
    addChild(horiz, _("This contains framey-type shietznite."));

  // For the nonce, return codes from and to the API should
  //  return "0" for failure and "1" for success.
  return 1;
}

// ===========================================================================
//
//  EXAMPLEB: "ExampleBWnd::startURLDownload"
//
void ExampleBWnd::startURLDownload() {
  // In this function, we are loading the url given to us in the edit field and
  // hoping that the XML parser can make SOME sort of sense out of whatever document
  // we can find.
  OutputDebugString(_("\n\n*************************************\n"));
  OutputDebugString(_("Now I am refreshing the XML URL....\n"));

  xmlurledit->getBuffer(myXMLURLBuffer, EXB_XML_BUFFSIZE);

  OutputDebugString(_("\n And this is the url I'm going to try to be getting: "));
  OutputDebugString(myXMLURLBuffer);

/*
  // WIP:   If "myXMLURLBuffer" could be a STRING, we could do either of these.  Which are both much cooler.

  myXMLURLBuffer = xmlurledit->getBuffer( );  

  OutputDebugString("\n 2) But this is the url I'm going to try to be getting: " );
  OutputDebugString(myXMLURLBuffer);

  myXMLURLBuffer = *xmlurledit; 

  OutputDebugString("\n 3) But this is the url I'm going to try to be getting: " );
  OutputDebugString(myXMLURLBuffer);

  // WIP:   This is a bit more along the lines of the interface that
  //        EditWnd should have.  Be patient with Dr Meeeeg: we have
  //        to freeze all those little changes like that until after
  //        we ship primary functionality to Beta standards.
*/

  // So.  Anyhow.  We've got an url.  Start downloading it.
  // *** UrlGrab is an interface to an asynchronous HTTP downloader.

  // First, if we're currently downloading anything, knock that shit off right now.
  if (myUrlGrab != NULL) {
    // yes, deleting terminates the download.
    // what, do you think we're morons or something?
    delete myUrlGrab; 
    myUrlGrab = NULL;
  }
  // don't forget to delete the tempfile.
  api->fileRemove(xmlTempFilename);
  // okay, so, urlgrab is off...
  myUrlGrab = new UrlGrab(myXMLURLBuffer, xmlTempFilename );
  // ...and running.
  pollDownloadState();  // "mommy are we THERE yet?"


  //
  // Okay, anyhow, enough talking.  Let's do it!  XML AHOY!
  //
  OutputDebugString(_("\n*** DONE refreshing the XML URL....\n"));
}

// ===========================================================================
//
//  EXAMPLEB: "ExampleBWnd::pollDownloadState"
//
void ExampleBWnd::pollDownloadState() {
  // not testing for this would be "bad," right?
  if (myUrlGrab != NULL) {
    int urlgrabstate = myUrlGrab->run();  
    //
    ExampleBDownloadStates newDownloadState;
    // While this is a lame way to use the return value from run()
    // as the current state of our download machine, it is an excellent
    // means of showing what each return value of run() represents.
    switch (urlgrabstate) {
      case -1:
        newDownloadState = EXB_DOWNLOAD_ERROR;
        break;
      case  0:
        newDownloadState = EXB_DOWNLOAD_WAIT;
        break;
      case  1:
        newDownloadState = EXB_DOWNLOAD_READY;
        break;
      default:
        // Since we already have the internal concept for an error,
        // it makes it easy to handle other things... like unexpected
        // return codes from functions.
        newDownloadState = EXB_DOWNLOAD_ERROR;
        break;
    }
    // or something.  :P
    
    // If our status has changed, reflect that fact.
    if (myDownloadState != newDownloadState) {
      // Wow!  TWO switch statements in ONE function!  I never seen THAT before, man!
      switch (newDownloadState) {
        case EXB_DOWNLOAD_ERROR:
          // Put up the error box into the display region.
          displayDownloadError();
          break;
        case EXB_DOWNLOAD_WAIT:
          // Put up the 'Please wait' box into the display region
          displayDownloadWait();
          break;
        case EXB_DOWNLOAD_READY:
          // Delete the current UrlGrabber.
          delete myUrlGrab;
          myUrlGrab = NULL;
          // Parse the file.  Setup the display.
          displayXMLData();
          break;
      }
      myDownloadState = newDownloadState;
    }
  }
}

// Put up the error box into the display region.
void ExampleBWnd::displayDownloadError() {
  displayText( _("Error downloading file from url.  You bozo.") );
}
// Put up the 'Please wait' box into the display region
void ExampleBWnd::displayDownloadWait() {
  displayText( _("Please wait while the file is downloaded.....") );
}
// Parse the file.  Setup the display.
void ExampleBWnd::displayXMLData() {
  // Clear the display
  displayText("");
  //
  // Now we setup tree items with the magic of OOP-XML!
  //
  // *** Read these, damn you.
  //
  // Firstly, clear whatever was in the tree, previously.
  myFrameTreeWnd->deleteAllItems();
  // Create a new single root node which will hold the constructed node-tree.
  ExampleBFrameXmlBase * rootNode = new ExampleBFrameXmlBase(myFrameTreeWnd, this);
  // Give it an ugly, generic label.
  rootNode->setLabel("-------");
  // Add it into the treeWnd
  myFrameTreeWnd->addTreeItem(rootNode, NULL, FALSE, TRUE);
  // Then instantiate and construct a new XmlManager to control the parsing.
  XmlManager<ExampleBFrameXmlBase> myXmlManager(rootNode);
  // Then, simply enough, we register the callback and read the file.
  XmlReader::registerCallback("*",&myXmlManager);
  // How can anything in the world be easier than this?    :P  God, I love OOP.
  XmlReader::loadFile(xmlTempFilename);
}
// Set the text in the display field.
void ExampleBWnd::displayText(const char * text) {
  myFrameTextBar->setName(text);
}


//
// But, but...  This file just looks so empty without a comment at the end.